@@ -8,7 +8,7 @@ PDF Reference, third edition, Version 1.4, 2001. ISBN 0-201-75839-3.
 PDF Reference, sixth edition, Version 1.7, 2006.
 """
 
-from enum import IntFlag
+from enum import IntFlag, auto
 from typing import Dict, Tuple
 
 
@@ -224,6 +224,7 @@ class ImageAttributes:
     BITS_PER_COMPONENT = "/BitsPerComponent"  # integer, required
     COLOR_SPACE = "/ColorSpace"  # name, required
     DECODE = "/Decode"  # array, optional
+    INTENT = "/Intent"  # string, optional
     INTERPOLATE = "/Interpolate"  # boolean, optional
     IMAGE_MASK = "/ImageMask"  # boolean, optional
     MASK = "/Mask"  # 1-bit image mask stream
@@ -543,7 +544,7 @@ class PageLabelStyle:
 
 
 class AnnotationFlag(IntFlag):
-    """See 12.5.3 "Anntation Flags"."""
+    """See 12.5.3 "Annotation Flags"."""
 
     INVISIBLE = 1
     HIDDEN = 2
@@ -584,3 +585,12 @@ PDF_KEYS = (
     TypArguments,
     TypFitArguments,
 )
+
+
+class ImageType(IntFlag):
+    NONE = 0
+    XOBJECT_IMAGES = auto()
+    INLINE_IMAGES = auto()
+    DRAWING_IMAGES = auto()
+    ALL = XOBJECT_IMAGES | INLINE_IMAGES | DRAWING_IMAGES
+    IMAGES = ALL  # for consistency with ObjectDeletionFlag
